"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTsProjectSourceFile = exports.isModulePathsMatch = exports.getOutputFile = exports.createParsedCommandLineForProgram = void 0;
const path_1 = __importDefault(require("path"));
/* ****************************************************************************************************************** */
// region: TS Helpers
/* ****************************************************************************************************************** */
/**
 * Generates a ParsedCommandLine for Program
 */
function createParsedCommandLineForProgram(tsInstance, program) {
    const compilerOptions = program.getCompilerOptions();
    const maybePcl = compilerOptions.configFilePath
        ? tsInstance.getParsedCommandLineOfConfigFile(compilerOptions.configFilePath, {}, tsInstance.sys)
        : void 0;
    return (maybePcl !== null && maybePcl !== void 0 ? maybePcl : tsInstance.parseJsonConfigFileContent({ files: program.getRootFileNames(), compilerOptions }, tsInstance.sys, program.getCurrentDirectory()));
}
exports.createParsedCommandLineForProgram = createParsedCommandLineForProgram;
/**
 * Determine output file path for source file
 */
function getOutputFile(context, fileName) {
    var _a;
    const { tsInstance, parsedCommandLine, outputFileNamesCache, program, compilerOptions } = context;
    if (outputFileNamesCache.has(fileName))
        return outputFileNamesCache.get(fileName);
    let res = void 0;
    const [tsMajor, tsMinor] = tsInstance.versionMajorMinor.split(".");
    // TS 3.7+ supports getOutputFileNames
    if (isTsProjectSourceFile(context, fileName) && (+tsMajor >= 4 || +tsMinor >= 7)) {
        try {
            res = tsInstance.getOutputFileNames(parsedCommandLine, fileName, (_a = tsInstance.sys) === null || _a === void 0 ? void 0 : _a.useCaseSensitiveFileNames)[0];
        }
        catch (e) {
            console.warn(`Failed to resolve output name for ${fileName}. Please report a GH issue at: ` +
                `https://github.com/LeDDGroup/typescript-transform-paths/issues`);
            debugger;
        }
    }
    if (!res)
        res = manualResolve();
    outputFileNamesCache.set(fileName, res);
    return tsInstance.normalizePath(res);
    function manualResolve() {
        var _a;
        const srcDir = program.getCommonSourceDirectory();
        const destDir = (_a = compilerOptions.outDir) !== null && _a !== void 0 ? _a : srcDir;
        return path_1.default.resolve(destDir, path_1.default.relative(srcDir, fileName));
    }
}
exports.getOutputFile = getOutputFile;
/**
 * Determine if moduleName matches config in paths
 */
function isModulePathsMatch(context, moduleName) {
    const { matchPatternOrExact, getOwnKeys } = context.tsInstance;
    return !!matchPatternOrExact(getOwnKeys(context.compilerOptions.paths), moduleName);
}
exports.isModulePathsMatch = isModulePathsMatch;
function isTsProjectSourceFile(context, filePath) {
    const { tsInstance, program } = context;
    return !!program.getRootFileNames().find((f) => tsInstance.normalizePath(filePath) === tsInstance.normalizePath(f));
}
exports.isTsProjectSourceFile = isTsProjectSourceFile;
// endregion
